@@ -28,15 +28,14 @@
 #include "gsm-process-helper.h"
 #include "gsm-util.h"
 
-#define GSM_DEFAULT_SESSION "gnome"
-
 #define GSM_KEYFILE_SESSION_GROUP "GNOME Session"
 #define GSM_KEYFILE_RUNNABLE_KEY "IsRunnableHelper"
 #define GSM_KEYFILE_FALLBACK_KEY "FallbackSession"
 #define GSM_KEYFILE_REQUIRED_KEY "Required"
 #define GSM_KEYFILE_DEFAULT_KEY "DefaultApps"
 
-#define GSM_RUNNABLE_HELPER_TIMEOUT 500 /* ms */
+/* See https://bugzilla.gnome.org/show_bug.cgi?id=641992 for discussion */
+#define GSM_RUNNABLE_HELPER_TIMEOUT 3000 /* ms */
 
 /* This doesn't contain the required components, so we need to always
  * call append_required_apps() after a call to append_default_apps(). */
@@ -361,12 +360,16 @@ find_valid_session_keyfile (const char *session)
 }
 
 static GKeyFile *
-get_session_keyfile (const char *session)
+get_session_keyfile (const char *session,
+                     char      **actual_session,
+                     gboolean   *is_fallback)
 {
         GKeyFile *keyfile;
         gboolean  session_runnable;
         char     *value;
 
+        *actual_session = NULL;
+
         g_debug ("fill: *** Getting session '%s'", session);
 
         keyfile = find_valid_session_keyfile (session);
@@ -389,8 +392,12 @@ get_session_keyfile (const char *session)
                 session_runnable = check_required_components (keyfile);
         }
 
-        if (session_runnable)
+        if (session_runnable) {
+                *actual_session = g_strdup (session);
+                if (is_fallback)
+                        *is_fallback = FALSE;
                 return keyfile;
+        }
 
         g_debug ("fill: *** Session is not runnable");
 
@@ -402,8 +409,11 @@ get_session_keyfile (const char *session)
         g_key_file_free (keyfile);
         keyfile = NULL;
 
-        if (!IS_STRING_EMPTY (value))
-                keyfile = get_session_keyfile (value);
+        if (!IS_STRING_EMPTY (value)) {
+                if (is_fallback)
+                        *is_fallback = TRUE;
+                keyfile = get_session_keyfile (value, actual_session, NULL);
+        }
         g_free (value);
 
         return keyfile;
@@ -415,20 +425,23 @@ gsm_session_fill (GsmManager  *manager,
                   const char  *session)
 {
         GKeyFile *keyfile;
+        gboolean is_fallback;
+        char *actual_session;
 
         if (override_autostart_dirs != NULL) {
                 load_override_apps (manager, override_autostart_dirs);
                 return TRUE;
         }
 
-        if (IS_STRING_EMPTY (session))
-                session = GSM_DEFAULT_SESSION;
-
-        keyfile = get_session_keyfile (session);
+        keyfile = get_session_keyfile (session, &actual_session, &is_fallback);
 
         if (!keyfile)
                 return FALSE;
 
+        _gsm_manager_set_active_session (manager, actual_session, is_fallback);
+
+        g_free (actual_session);
+
         load_standard_apps (manager, keyfile);
 
         g_key_file_free (keyfile);
